@@ -29,10 +29,12 @@ from PyQt5.QtCore import (Qt, QVariant, QModelIndex, pyqtSignal, pyqtSlot)
 from PyQt5 import QtGui, QtWidgets
 from PyQt5.QtWidgets import QFileDialog
 import qtawesome
+import re
 import textwrap
 import time
 import twisted.internet.defer
 import twisted.internet.task
+import uuid
 import weakref
 
 # See file COPYING in this source tree
@@ -88,6 +90,10 @@ class TooManyMetaSignalsError(Exception):
     pass
 
 
+class SimultaneousTransactionsError(Exception):
+    pass
+
+
 @attr.s
 class Configuration:
     set_frame = attr.ib()
@@ -469,6 +475,11 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
             metas=metas,
         )
 
+        self.nv_from_uuid = {
+            nv.parameter_uuid: nv
+            for nv in self.all_nv()
+        }
+
     def terminate(self):
         self.cancel_cyclic_read_all()
 
@@ -545,75 +556,85 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
 
         if not background:
             self.activity_started.emit('{}...'.format(activity))
-        d = twisted.internet.defer.Deferred()
-        d.callback(None)
 
-        def handle_frame(frame, signals, enumerator):
-            if read:
-                d.addCallback(
-                    lambda _, enumerator=enumerator: self.protocol.read_multiple(
-                        nv_signals=signals,
-                        meta=enumerator,
-                        priority=epyqlib.twisted.nvs.Priority.user,
-                        passive=True,
-                        all_values=True,
+        try:
+            d = twisted.internet.defer.Deferred()
+            d.callback(None)
+
+            def handle_frame(frame, signals, enumerator):
+                if read:
+                    d.addCallback(
+                        lambda _, enumerator=enumerator: self.protocol.read_multiple(
+                            nv_signals=signals,
+                            meta=enumerator,
+                            priority=epyqlib.twisted.nvs.Priority.user,
+                            passive=True,
+                            all_values=True,
+                        )
                     )
-                )
-            elif frame.read_write.min <= 0:
-                not_none_signals = {}
-                for signal in signals:
-                    values_key = (signal, enumerator)
-                    if values_key in values:
-                        value = values[values_key]
-                    elif enumerator == MetaEnum.value:
-                        value = signal.value
-                    else:
-                        value = getattr(signal.meta, enumerator.name).value
+                elif frame.read_write.min <= 0:
+                    not_none_signals = {}
+                    for signal in signals:
+                        values_key = (signal, enumerator)
+                        if values_key in values:
+                            value = values[values_key]
+                        elif enumerator == MetaEnum.value:
+                            value = signal.value
+                        else:
+                            value = getattr(signal.meta, enumerator.name).value
 
-                    if value is not None:
-                        not_none_signals[signal] = value
+                        if value is not None:
+                            not_none_signals[signal] = value
 
-                if len(not_none_signals) == 0:
+                    if len(not_none_signals) == 0:
+                        return
+
+                    d.addCallback(
+                        lambda _, enumerator=enumerator, not_none_signals=not_none_signals: self.protocol.write_multiple(
+                            nv_signals=not_none_signals,
+                            meta=enumerator,
+                            priority=epyqlib.twisted.nvs.Priority.user,
+                            passive=True,
+                            all_values=True,
+                        )
+                    )
+                else:
                     return
 
-                d.addCallback(
-                    lambda _, enumerator=enumerator, not_none_signals=not_none_signals: self.protocol.write_multiple(
-                        nv_signals=not_none_signals,
-                        meta=enumerator,
-                        priority=epyqlib.twisted.nvs.Priority.user,
-                        passive=True,
-                        all_values=True,
+                if callback is not None:
+                    d.addCallback(callback)
+
+            if only_these is None:
+                only_these = self.all_nv()
+
+            frames = set(nv.frame for nv in only_these)
+            for frame in frames:
+                signals = tuple(nv for nv in only_these
+                                if nv.frame is frame)
+
+                if read:
+                    data = tuple(
+                        signal.some_packable_value()
+                        for signal in frame.signals
                     )
-                )
-            else:
-                return
+                    frame.update_from_signals(data=data)
+                else:
+                    frame.update_from_signals()
 
-            if callback is not None:
-                d.addCallback(callback)
-
-        if only_these is None:
-            only_these = self.all_nv()
-
-        frames = set(nv.frame for nv in only_these)
-        for frame in frames:
-            signals = tuple(nv for nv in only_these
-                            if nv.frame is frame)
-
-            frame.update_from_signals()
-            for enumerator in meta:
-                handle_frame(
-                    frame=frame,
-                    signals=signals,
-                    enumerator=enumerator,
-                )
-
-        if not background:
-            d.addCallback(epyqlib.utils.twisted.detour_result,
-                          self.activity_ended.emit,
-                          'Finished {}...'.format(activity.lower()))
-            d.addErrback(epyqlib.utils.twisted.detour_result,
-                         self.activity_ended.emit,
-                         'Failed while {}...'.format(activity.lower()))
+                for enumerator in meta:
+                    handle_frame(
+                        frame=frame,
+                        signals=signals,
+                        enumerator=enumerator,
+                    )
+        finally:
+            if not background:
+                d.addCallback(epyqlib.utils.twisted.detour_result,
+                              self.activity_ended.emit,
+                              'Finished {}...'.format(activity.lower()))
+                d.addErrback(epyqlib.utils.twisted.detour_result,
+                             self.activity_ended.emit,
+                             'Failed while {}...'.format(activity.lower()))
 
         return d
 
@@ -679,6 +700,7 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
                     maximum=child.meta.maximum.get_human_value(for_file=True),
                     readable=not child.is_write_only(),
                     writable=not child.is_read_only(),
+                    parameter_uuid=child.parameter_uuid,
                 )
             elif include_secrets:
                 def format_float(value):
@@ -696,6 +718,7 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
                     factory_default=format_float(value=0),
                     minimum=format_float(value=child.min),
                     maximum=format_float(value=child.max),
+                    parameter_uuid=child.parameter_uuid,
                 )
             else:
                 continue
@@ -729,13 +752,9 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
             ),
         )
         only_in_file = {
-            parameter.name
-            for parameter in parameter_nodes
-        }
-        only_in_file = {
-            (name, meta)
-            for name, meta in itertools.product(
-                only_in_file,
+            (parameter, meta)
+            for parameter, meta in itertools.product(
+                parameter_nodes,
                 epyqlib.nv.MetaEnum,
             )
         }
@@ -743,11 +762,14 @@ class Nvs(TreeNode, epyqlib.canneo.QtCanListener):
         d = collections.defaultdict(list)
         for parameter in parameter_nodes:
             d[parameter.name].append(parameter)
+            d[parameter.parameter_uuid].append(parameter)
 
         for child in self.all_nv():
             name = child.fields.name
 
-            parameters = d.get(name, [])
+            parameters = d.get(child.parameter_uuid, [])
+            if len(parameters) == 0:
+                parameters = d.get(name, [])
 
             not_found_format = (
                 "Nv value named '{}' ({{}}) not found when loading "
@@ -1471,6 +1493,7 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
         color = QtGui.QColor(palette.windowText().color())
         color.setAlphaF(0.3)
         self.stale_foreground = color
+        self.edit_locally = False
 
     def all_nv(self):
         return self.root.all_nv()
@@ -1573,7 +1596,13 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
     def check_range_changed(self, state):
         self.check_range = state == Qt.Checked
 
+    def edit_locally_changed(self, state):
+        self.edit_locally = state == Qt.Checked
+
     def start_transaction(self):
+        if self.transaction_actions is not None:
+            raise SimultaneousTransactionsError(self.transaction_actions)
+
         self.transaction_actions = collections.defaultdict(list)
 
     def submit_transaction(self):
@@ -1643,7 +1672,10 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
 
         await self._submit_transaction()
 
-    def setData(self, index, data, role=None):
+    def setData(self, index, data, role=None, check_range=None):
+        if check_range is None:
+            check_range = self.check_range
+
         column = index.column()
 
         if data == '':
@@ -1654,10 +1686,10 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
                 node = self.node_from_index(index)
                 valid = node.check_data(
                     data,
-                    check_range=self.check_range,
+                    check_range=check_range,
                 )
                 if valid:
-                    node.scratch.set_human_value(data, check_range=self.check_range)
+                    node.scratch.set_human_value(data, check_range=check_range)
                     node.fields.scratch = node.scratch.full_string
         elif column in self.meta_columns:
             if role == Qt.EditRole:
@@ -1669,35 +1701,47 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
                 valid = node.check_meta(
                     data,
                     meta=meta,
-                    check_range=self.check_range,
+                    check_range=check_range,
                 )
 
                 if valid:
                     nodes = (node,)
 
-                    if self.transaction_actions is None:
-                        callback = functools.partial(
-                            self.update_signals,
+                    if self.edit_locally:
+                        self.update_signals(
+                            arg=[
+                                {node.status_signal: data},
+                                meta,
+                            ],
                             only_these=nodes,
                         )
-
-                        d = self.root.write_all_to_device(
-                            only_these=nodes,
-                            values={(node, meta): node.calc_human_value(data)},
-                            callback=callback,
-                            meta=(meta,),
-                        )
-                        d.addErrback(epyqlib.utils.twisted.catch_expected)
-                        d.addErrback(epyqlib.utils.twisted.errbackhook)
                     else:
-                        self.transaction_actions[meta].append(
-                            TransactionAction(
+                        human_value = node.calc_human_value(data)
+                        values = {(node, meta): human_value}
+
+                        if self.transaction_actions is None:
+                            callback = functools.partial(
+                                self.update_signals,
                                 only_these=nodes,
-                                values={(node, meta): node.calc_human_value(data)},
-                                callback=self.update_signals,
-                                meta=meta,
-                            ),
-                        )
+                            )
+
+                            d = self.root.write_all_to_device(
+                                only_these=nodes,
+                                values=values,
+                                callback=callback,
+                                meta=(meta,),
+                            )
+                            d.addErrback(epyqlib.utils.twisted.catch_expected)
+                            d.addErrback(epyqlib.utils.twisted.errbackhook)
+                        else:
+                            self.transaction_actions[meta].append(
+                                TransactionAction(
+                                    only_these=nodes,
+                                    values=values,
+                                    callback=self.update_signals,
+                                    meta=meta,
+                                ),
+                            )
 
         return False
 
@@ -1861,13 +1905,9 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
             ),
         )
         only_in_file = {
-            parameter.name
-            for parameter in parameter_nodes
-        }
-        only_in_file = {
-            (name, meta)
-            for name, meta in itertools.product(
-                only_in_file,
+            (parameter, meta)
+            for parameter, meta in itertools.product(
+                parameter_nodes,
                 epyqlib.nv.MetaEnum,
             )
         }
@@ -1875,6 +1915,7 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
         d = collections.defaultdict(list)
         for parameter in parameter_nodes:
             d[parameter.name].append(parameter)
+            d[parameter.parameter_uuid].append(parameter)
 
         skip_nodes = (
             self.root.access_level_node,
@@ -1899,7 +1940,9 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
 
                 name = child.fields.name
 
-                parameters = d.get(name, [])
+                parameters = d.get(child.parameter_uuid, [])
+                if len(parameters) == 0:
+                    parameters = d.get(name, [])
 
                 not_found_format = (
                     "Nv value named '{}' ({{}}) not found when loading "
@@ -1912,7 +1955,7 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
                     index = self.index_from_node(child)
 
                     for meta in MetaEnum:
-                        only_in_file.discard((name, meta))
+                        only_in_file.discard((parameter, meta))
 
                         v = getattr(parameter, meta.name)
                         if v is not None:
@@ -1922,6 +1965,7 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
                                 ),
                                 v,
                                 Qt.EditRole,
+                                check_range=False,
                             )
                         else:
                             logger.warning(not_found_format.format(meta.name))
@@ -1936,9 +1980,9 @@ class NvModel(epyqlib.pyqabstractitemmodel.PyQAbstractItemModel):
                         "value set".format(name),
                     )
 
-            for name, meta in sorted(only_in_file):
+            for parameter, meta in sorted(only_in_file):
                 logger.warning("Unrecognized NV value named '{}' ({}) found when loading "
-                      "from value set".format(name, meta.name))
+                      "from value set".format(parameter.name, meta.name))
 
 
 if __name__ == '__main__':
